#include "textures.h"
#include <assert.h>

void loadTextures(char *filename, GLuint textureID);

textures_t *textures_init(void) {
	textures_t *t = (textures_t*) malloc(sizeof(textures_t));
	// Generate 10 textures
	glGenTextures(NUM_TEXTURES, t->texture);
	loadTextures("ground.bmp", t->texture[TEXTURE_GROUND]);
	return t;
}

void reverse(char* data,int size) {
  int i;
  char temp;
  // on BIG ENDIAN architecture don't reverse byte order
#ifdef _WIN32
  return;
#endif
#ifdef BIG_ENDIAN
  return;
#endif
  for (i=0;i<size/2;i++)
  {
    temp=data[i];
    data[i]=data[size-i-1];
    data[size-i-1]=temp;
  }
}

// quick and dirty bitmap loader...for 24 bit bitmaps with 1 plane only.  
// See http://www.dcs.ed.ac.uk/~mxr/gfx/2d/BMP.txt for more info.
int ImageLoad(char *filename, Image *image) {
    FILE *file;
    unsigned long size;                 // size of the image in bytes.
    unsigned long i;                    // standard counter.
    unsigned short int planes;          // number of planes in image (must be 1) 
    unsigned short int bpp;             // number of bits per pixel (must be 24)
    char temp;                          // temporary color storage for bgr-rgb conversion.
    char k[4];

    // make sure the file is there.
    if ((file = fopen(filename, "rb"))==NULL) {
		printf("File Not Found : %s\n",filename);
		return 0;
    }
    
    // seek through the bmp header, up to the width/height:
    fseek(file, 18, SEEK_CUR);

    // read the width
    if ((i = (unsigned long) fread(k, 4, 1, file)) != 1) {
		printf("Error reading width from %s.\n", filename);
		return 0;
    }

    reverse(k,4);
    image->sizeX=*(int*)k;
    
    printf("Width of %s: %lu\n", filename, image->sizeX);
    
    // read the height 
    if ((i = (unsigned long)fread(k, 4, 1, file)) != 1) {
		printf("Error reading height from %s.\n", filename);
		return 0;
    }

    reverse(k,4);
    image->sizeY=*(int*)k;

    printf("Height of %s: %lu\n", filename, image->sizeY);
    
    // calculate the size (assuming 24 bits or 3 bytes per pixel).
    size = image->sizeX * image->sizeY * 3;

    // read the planes
    if ((fread(k, 2, 1, file)) != 1) {
		printf("Error reading planes from %s.\n", filename);
		return 0;
    }
    reverse(k,2);
    planes=*(int*)k;
    planes=1; // :)

    if (planes != 1) {
		printf("Planes from %s is not 1: %u\n", filename, planes);
		return 0;
    }
    // read the bpp
    if ((i = (unsigned long)fread(&bpp, 2, 1, file)) != 1) {
		printf("Error reading bpp from %s.\n", filename);
		return 0;
    }
    bpp=24;
    if (bpp != 24) {
		printf("Bpp from %s is not 24: %u\n", filename, bpp);
		return 0;
    }
	
    // seek past the rest of the bitmap header.
    fseek(file, 24, SEEK_CUR);

    // read the data. 
    image->data = (char *) malloc(size);
    if (image->data == NULL) {
		printf("Error allocating memory for color-corrected image data");
		return 0;	
    }

    if ((i = (unsigned long)fread(image->data, size, 1, file)) != 1) {
		printf("Error reading image data from %s.\n", filename);
		return 0;
    }

    for (i=0;i<size;i+=3) { // reverse all of the colors. (bgr -> rgb)
		temp = image->data[i];
		image->data[i] = image->data[i+2];
		image->data[i+2] = temp;
    }
    
    // we're done.
    return 1;
}

void loadTextures(char *filename, GLuint textureID) {	
    // Load Texture
    Image *image1;
    
	 assert(textureID < NUM_TEXTURES);

    // allocate space for texture
    image1 =  (Image *) malloc(sizeof(Image));
    if (image1 == NULL) {
		printf("Error allocating space for image");
    }

    if (!ImageLoad(filename, image1)) {
		printf("file not found ?\n");
    }        

    // Create Texture	
    glBindTexture(GL_TEXTURE_2D, textureID);   // 2d texture (x and y size)

    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); // scale linearly when image bigger than texture
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); // scale linearly when image smalled than texture

    // 2d texture, level of detail 0 (normal), 3 components (red, green, blue), x size from image, y size from image, 
    // border 0 (normal), rgb color data, unsigned byte data, and finally the data itself.
    glTexImage2D(GL_TEXTURE_2D, 0, 3, image1->sizeX, image1->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, image1->data);
}

